<?php
/*
 * © Copyright 2010 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Translate Templates
 *
 * @package I2CE
 * @subpackage Core
 * @author Carl Leitner <litlfred@ibiblio.org>
 * @copyright Copyright &copy; 2008, 2008 IntraHealth International, Inc. 
 * @version 1.0
 */


$whoami = trim(`whoami`);
$tmp_dir_base ="/home/" . trim($whoami) . '/ihris-release';
$base_dir = getcwd();
$i2ce_dir = realpath(dirname(__FILE__) . DIRECTORY_SEPARATOR . '..') . DIRECTORY_SEPARATOR;

$locales = 'fr,sw,pt,pt_BR,et,it';

$ignore_filenames = array('/^.bzr/');
$ignore_filepaths = array();

$search_dirs = array();
$booleans['prompt'] = true;
$booleans['only-new'] = false;
$booleans['skip-updates'] = false;
$usage[] = 
    "[--prompt=T/F] set to false to never prompt on actions\n".
    "\tDefaults to true\n"; 
$usage[] = 
    "[--only-new=T/F] set to true to only create modules that have changed since the previous release\n".
    "\tDefaults to false\n"; 
$usage[] = 
    "[--skip-updates=T/F] set to true skip updating branches\n".
    "\tDefaults to false\n"; 

require_once( $i2ce_dir. DIRECTORY_SEPARATOR . 'tools' . DIRECTORY_SEPARATOR . 'base.php');
require_once( $i2ce_dir. DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'I2CE_Validate.php');

getSuPassword();



if ($booleans['prompt'] && !simple_prompt("use $tmp_dir_base to store release branches and debian packaging?")) {
    while( true) {
        $tmp_dir_base = ask("What directory should I use");
        if (is_dir($tmp_dir_base)) {
            break;
        } else {
            if (simple_prompt("Create $tmp_base_dir?")) {
                exec ("mkdir -p $tmp_base_dir");
                break;
            }
        }
    }
}
$deb_src_dir = $tmp_dir_base .  '/deb/src';
$deb_bin_dir = $tmp_dir_base .  '/deb/bin';


$branches_dir = $tmp_dir_base . DIRECTORY_SEPARATOR . 'branch';
if (!is_dir($deb_src_dir)) {
    if (!$booleans['prompt'] || simple_prompt("Create $deb_src_dir?")) {
        exec ("mkdir -p $deb_src_dir");
    } else {
        die("No place to store stuff");
    }
}
if (!is_dir($deb_bin_dir)) {
    if (!$booleans['prompt'] || simple_prompt("Create $deb_bin_dir?")) {
        exec ("mkdir -p $deb_bin_dir");
    } else {
        die("No place to store stuff");
    }
}

if (!is_dir($branches_dir)) {
    if (!$booleans['prompt'] || simple_prompt("Create $branches_dir?")) {
        exec ("mkdir -p $branches_dir");
    } else {
        die("No place to store branches");
    }
}



$site_dirs = array();
$uncommitted = null;
$top_dirs = array();
$t_dirs = glob("*",GLOB_ONLYDIR);
foreach ($t_dirs as $i=>$dir) {
    $ret = 0;
    $out = array();
    $info = @exec("bzr info $dir 2> /dev/null",$out,$ret);
    if ($ret !== 0) {
        unset($t_dirs[$i]);
        continue;
    }
    $ret = 0;
    $out = array();
    $info = @exec("bzr status -S $dir 2> /dev/null",$out,$ret);
    if (count($out)>0) {
        I2CE::raiseError("Warning $dir has uncommitted changes");
    }
    $dir = realpath($dir);
    $branch = basename($dir);
    $branch_dir = "$branches_dir/$branch";
    if (!is_dir($branch_dir)) {
        $cmd = "bzr branch --bind $dir $branch_dir";    
        exec($cmd);                    
    } else {
        if (!$booleans['skip-updates']) {
            $cmd = "bzr update $branch_dir";    
            exec($cmd);                    
        }
    }
    $top_dirs[]  = $branch_dir;
    $site_dirs = array_merge($site_dirs, glob("$branch_dir/sites/*",GLOB_ONLYDIR));    
}

$search_dirs = array_merge($site_dirs,$top_dirs);



if ($booleans['prompt'] && !simple_prompt("Work with the following directories:\n\t" . implode("\n\t",$search_dirs) ."\n")) {
    die("Stopping\n");
}


function vers_compare($vers1,$vers2) {
    if (I2CE_Validate::checkVersion($vers1,'=',$vers2)) { 
        return 0;
    } 
    if (I2CE_Validate::checkVersion($vers1,"<",$vers2)) {
        return 1;
    } else {
        return -1;
    }
}


$releases = array();
$change_logs = array();
$change_files = array();
$all_changes = null;
foreach ($top_dirs as $dir) {
    if (! ($pcre = trim(`which pcregrep`))) {
        die("Please do: sudo apt-get install pcregrep\n");
    }
    I2CE::raiseError("Finding release tags for $dir");
    $cmd = "bzr log --line $dir | $pcre '" . '\\{[0-9\\.]+-release\\}'  . "'";
    $out = array();
    exec($cmd, $out,$ret);
    $t_releases = array();
    foreach ($out as $line) {
        if (preg_match('/^([0-9\\.]+):.*?\\{([0-9\\.]+)-release\}/',$line,$matches)) {
            $t_releases[] = array($matches[1],$matches[2]); //revno,release
        }
    }
    if (count($t_releases) == 0) {
        I2CE::raiseError("No X.Y.Z-release tags for $dir");
        continue;
    }

    list($revno,$release) = $t_releases[0];
    $t_releases[] = array(1,'0.0');

    $cmd = "grep  -P 'I2CEConfiguration\s+name' $dir/*xml ";
    if (!preg_match('/name=["\'](.*?)["\']/' ,trim( `$cmd`),$matches)) {
        I2CE::raiseError("Could not find a module in $dir");
        continue;
    }
    $mod =$matches[1];
    $cmd = "grep  '/version' $dir/*xml | head -1 ";
    $out = trim( `$cmd`);
    if (!preg_match('/([0-9\.]+)/' ,$out,$matches2)) {
        I2CE::raiseError("Could not find a version in $dir");
        continue;
    }
    $version = $matches2[1];

    $version_components = array_pad(explode(".",$version),3,0);
    $sub_major_version = implode(".", array_slice($version_components,0,2));
    $minor_version = implode(".", array_slice($version_components,0,3));


    $cmd = "bzr log --long -v -n1 -r ..$revno $dir";
    if ($minor_version != $release) {
        I2CE::raiseError("Warning $minor_version != $release");
    }
    $releases[$mod] = array('dir'=>basename($dir),'releases'=>$t_releases, 'revno'=>$revno,'release'=>$release,'version'=>$version,'sub_major_version'=>$sub_major_version);
    exec($cmd,$out,$ret);
    I2CE::raiseError("Processsing revisions upto release $release (rev $revno) in $dir for change log");
    $change_logs[$mod] = array();
    $in_rev = false;
//------------------------------------------------------------
//revno: 2297
//committer: Carl Leitner <litlfred@ibiblio.org>
//branch nick: 4.0-dev
//timestamp: Tue 2010-12-14 10:26:31 -0500
//message:
//   translate templates will try to solidify source and translated text in case the sites .po files have not been updated
//removed:
//   some/file/name
//modified:
//   tools/translate_templates.php
    $files = array();
    $message = '';
    $rev_no = false;
    $timestamp = false;
    $committer = false;
    $in_mod = false;
    $in_rnm = false;
    $bugs = '';
    $change_files[$mod]= array();
    $release = 'current'; //this should never actually be used as we should pick up the last release tag in the first revision
    foreach ($out as $line) {
        $line = rtrim($line);
        if (substr($line,0,10)  == '----------') {
            //new revision
            if ($rev_no) {
                $change_logs[$mod][$rev_no] = array('committer'=>$committer,'message'=>rtrim($message) . $bugs,'timestamp'=>$timestamp);
                foreach ($files as $file) {
                    if (!array_key_exists($file,$change_files[$mod])) {
                        $change_files[$mod ][ $file] = array();
                    }
                    if (!array_key_exists($release,$change_files[$mod][$file])) {
                        $change_files[$mod ][ $file][$release] = array();
                    }
                    $change_files[$mod][$file][$release][] = $rev_no;
                }
            }
            $files = array();
            $message = '';
            $bugs = '';
            $rev_no  =false;
            $timestamp = false;
            $committer = false;
            $in_mod = false;
            $in_rnm = false;
        } else if (preg_match('/^revno:\s+([0-9\.]+)/',$line,$matches)) {
            $rev_no = $matches[1];
            $in_msg = false;
            $in_mod = false;
            $in_rnm = false;
        } else if (preg_match('/^committer:\s+(.*?)\s*$/',$line,$matches)) {
            $committer = $matches[1];
            if (preg_match('/litlfred/', $committer)) {
                $committer = "Carl Leitner <litlfred@ibiblio.org>";
            }   else if (preg_match('/lduncan/', $committer)) {
                $committer = "Luke Duncan <lduncan@intrahealth.org>";
            }
            $in_msg = false;
            $in_mod = false;
            $in_rnm = false;
        } else if (preg_match('/^timestamp:\s+(.*?)\s*$/',$line,$matches)) {
            //input: Mon 2010-06-07 13:33:42 -0400
            //output: Mon, 07 June 2010 13:33:42 -0400
            // day-of-week, dd month yyyy hh:mm:ss +zzzz 
            $timestamp = $matches[1];
            $in_msg = false;
            $in_mod = false;
            $in_rnm = false;
        } else if (preg_match("/^branch nick:/",$line)) {
            $in_msg = false;
            $in_mod = false;
            $in_rnm = false;
            //ignore
        } else if (substr($line,0,13) == 'fixes bug(s):'){
            $in_msg = false;
            $in_mod = false;
            $in_rnm = false;
            $bugs .= "\n *   Fixes Bugs: " . trim(substr($line,14)) . "\n";
        } else if (preg_match("/^tags:/",$line)) {
            if (preg_match('/([0-9\\.]+)-release/',$line,$matches)) {
                $release = $matches[1];
            }
            $in_msg = false;
            $in_mod = false;
            $in_rnm = false;
            //ignore
        } else if (preg_match("/^message:/",$line)) {
            $in_msg = true;
            $in_mod = false;
            $in_rnm = false;
        } else if (preg_match("/^removed:/",$line) || preg_match("/^added:/",$line) || preg_match("/^modified:/",$line) || preg_match("/^kind changed:/",$line)) {
            $in_msg = false;
            $in_mod = true;
            $in_rnm = false;
        } else if (preg_match("/^renamed:/",$line)) {
            $in_msg = false;
            $in_mod = false;
            $in_rem = false;
            $in_add = false;
            $in_rnm = true;
        } else if (preg_match("/^[a-zA-Z][a-zA-Z\s]+:/",$line)) {
            $in_msg = true;
            $in_mod = false;
            $in_rem = false;
            $in_add = false;
            $in_rnm = false;
            I2CE::raiseError(   "Dont knwo what to do with:\n[[$line]]\n");
        } else if ($in_msg) {
            $line = implode("\n    ",explode("\n",wordwrap(trim($line))));
            if ($message) {
                $message .= "    " . $line ."\n";
            } else {
                $message  = "  * " . $line ."\n";
            }
        } else if ($in_mod) {
            if ($line) {
                $files[] = trim($line);
            }
        } else if ($in_rnm) {
            list($source,$dest) = explode("=>",$line);
            if ( ($source = trim($source)) && ($dest = trim($dest))) {
                $files[] = $dest;
                $files[] = $source;
            }
        } else {
            die ("Don't know what to do with (no-state):\n[[$line]]\n");
        }
        
        
    }
    if ($rev_no) {
        //get the (first = last)  revision
        $change_logs[$mod][$rev_no] = array('committer'=>$committer,'message'=>rtrim($message)  . $bugs,'timestamp'=>$timestamp);
        foreach ($files as $file) {
            if (!array_key_exists($file,$change_files[$mod])) {
                $change_files[$mod ][ $file] = array();
            }
            if (!array_key_exists($release,$change_files[$mod][$file])) {
                $change_files[$mod ][ $file][$release] = array();
            }
            $change_files[$mod][$file][$release][] = $rev_no;
        }

    }
}


if (!array_key_exists('I2CE',$releases)) {
    die("I2CE is not present\n");
}




function makePackageName($module) {
    $module = strtolower($module);
    $module = preg_replace('/--+/','-',preg_replace('/[^a-z0-9\\+]/','-',$module));
    $module = trim($module,'-');
    return   'ihris+'   . $module;
}

chdir($branches_dir);
getAvailableModules();

$package_names = array();
$mod_dirs =array();
$branch_modules = array();
foreach ($found_modules as $module=>$top_module) {
    $config_file = false;
    if ( ! $storage->setIfIsSet($config_file,"/config/data/$module/file")) {
        I2CE::raiseError( "No config file for $module -- Skipping");
        continue;
    }
    $package_names[$module] = makePackageName($module);
    $config_dir = rtrim(dirname($config_file),'/');
    if (!array_key_exists($top_module,$mod_dirs)) {
        $mod_dirs[$top_module] = array();
    }
    $mod_dirs[$top_module][$config_dir] = $module;
    
    //now get the top level module  in the branch
    if (!array_key_exists($top_module,$releases)) {
        //this is a site specific module
        if (!preg_match('/^(.*)-site.*$/',$top_module,$matches) || !array_key_exists($matches[1],$releases)) {
            I2CE::raiseError("Could not containing  module for $module");
            continue;
        }
        $b_module = $matches[1];
    } else {
        $b_module = $top_module;
    }
    $branch_modules[$module] = $b_module;
}



    
$versioned_files = array();


I2CE::raiseError("Getting all versioned files by module");
foreach ($found_modules as $module=>$top_module) {
    if (!array_key_exists($module,$branch_modules)) {
        continue;
    }
    if (!array_key_exists($top_module,$versioned_files)) {
        $versioned_files[$top_module] = array();
    }
    $v = array();
    $config_file = false;
    if ( ! $storage->setIfIsSet($config_file,"/config/data/$module/file")) {
        I2CE::raiseError( "No config file for $module -- Skipping");
        continue;
    }
    $config_dir = rtrim(dirname($config_file),'/');

    $t_config_file = false;
    if ( ! $storage->setIfIsSet($t_config_file,"/config/data/$top_module/file")) {
        I2CE::raiseError( "No config file for branch module $top_module -- Skipping");
        continue;
    }
    $t_config_dir = rtrim(dirname($t_config_file),'/');


    $dirs = array();
    //$ignore = array('MODULES','TEMPLATES','CONFIGS');
    $ignore = array('MODULES');
    foreach ($ignore as $type) {
        $dirs[$type] = array();
        $storage->setIfIsSet($dirs[$type],"/config/data/$module/paths/" . $type,true);
        foreach ($dirs[$type]  as $i=>&$s_d) {
            if ($s_d[0] != '/') {
                $s_d = realpath($config_dir . '/' . $s_d);
            } else {
                $s_d = realpath($s_d);
            } 
            if (!is_dir($s_d) || strpos($s_d,$config_dir) !== 0) { //check to see ensure is a subdir
                unset($dirs[$type][$i]);
                continue;
            }
        }
        unset($s_d);
    }

    $cmd = "bzr ls -R -V $config_dir";
    $v = array();
    @exec($cmd,$v);
    $t_dir_len = strlen($t_config_dir) + 1;
    foreach ($v as $i=>&$f) {
        $f = trim($f);
        foreach ($ignore_filepaths as $regexp) {
            if (preg_match($regexp,$f)) {
                unset($v[$i]);
                continue 2;
            }
        }
        $filename = basename($f);
        foreach ($ignore_filenames as $regexp) {
            if (preg_match($regexp,$filename)) {
                unset($v[$i]);
                continue 2;
            }
        }

        //first we ignore things that are not a subdirectory
        foreach ($ignore as $type) { 
            foreach ($dirs[$type] as $s_d) {
                if (strpos($f,$s_d) === 0) {
                    unset($v[$i]);
                    continue 3;
                }
            }
        }
        //now strip out begining of filename so we are only relative to the top level package module (e.g. a site module or a i2ce,irhis-common etc)
        $f = substr(trim($f),$t_dir_len);

        //now we check that it not within a site
        if ($f == 'sites' || strpos($f,'sites/') === 0) {
            unset($v[$i]);
            continue;
        }
        //if $top_module == $module we ignore the translations and a few other anothing directories
        //foreach (array('translations','tools','t','tests') as $ig) {
        foreach (array('translations','t','tests','maintainer_tools') as $ig) {
            if ($f == $ig || strpos($f, "$ig/") === 0) {
                unset($v[$i]);
                continue 2;
            }
        }
        
    }
    unset($f);
    $versioned_files[$top_module][$module] = $v;
}


///////////////////END GETTING VERSIOEND


I2CE::raiseError("Got all versioned files by module");

I2CE::raiseError("Producing changelog for modules");

$changed_modules = array();
foreach ($found_modules as $module=>$top_module) {
    if (!array_key_exists($module,$versioned_files[$top_module]) || count($versioned_files[$top_module][$module]) == 0) {
        I2CE::raiseError("Module $module has no versioned files");
        continue;
    }
    if (!array_key_exists($module,$branch_modules)) {
        I2CE::raiseError("Dont knwo which branch $module is in");
        continue;
    }
    $b_module = $branch_modules[$module];
    if (!array_key_exists($b_module,$change_logs) || count($change_logs[$b_module]) == 0) {
        I2CE::raiseError("Top module $top_module of module $module has no change logs");
        continue;
    }
    $version  = false;
    if (!$storage->setIfIsSet($version,"/config/data/$module/version")) {
        I2CE::raiseError("Cannot get version for $module.  Skipping");
        continue;
    }
    $changelog = '';
    $revs_by_release = array();
    $b_module == $branch_modules[$module];
    $prefix  ='';
    if ($b_module !== $top_module) {
        //it is a site module and there fore the prefix needs to change for the file
        $t_config_file = false;
        if ( ! $storage->setIfIsSet($t_config_file,"/config/data/$top_module/file")) {
            I2CE::raiseError( "No config file for top module $top_module -- Skipping");
            continue;
        }
        $t_config_dir = rtrim(dirname($t_config_file),'/');

        $b_config_file = false;
        if ( ! $storage->setIfIsSet($b_config_file,"/config/data/$b_module/file")) {
            I2CE::raiseError( "No config file for branch module $b_module -- Skipping");
            continue;
        }
        $b_config_dir = rtrim(dirname($b_config_file),'/');

        
        $prefix = substr($t_config_dir,strlen($b_config_dir)+1) .'/';
    }
    foreach ($releases[$b_module]['releases'] as $i=>$rev_release) {
        list($rev,$release) = $rev_release;
        $revs = array();
        foreach ($versioned_files[$top_module][$module] as $file) {
     
            if (!array_key_exists($prefix.$file,$change_files[$b_module])) {
                continue;
            }
            if (!array_key_exists($release,$change_files[$b_module][$prefix.$file])) {
                continue;
            }
            $revs = array_unique(array_merge($revs,$change_files[$b_module][$prefix.$file][$release]));
        }
        usort($revs,'vers_compare');
        if (count($revs) == 0) {
            continue;
        }
        $revs_by_release[$release] = $revs;
    }

    if (count($revs_by_release) == 0) {
        I2CE::raiseError("No revisions for $module");
        continue;
    }
    $changelog = '';
    $first = true;
    foreach ($revs_by_release as $release=>$revs) {
        //$change_logs[$top_mod][$rev_no] = array('committer'=>$committer,'rnm_files'=>$rnm_files,'mod_files' => $mod_files,'rem_files'=>$rem_files,'add_files'=>$add_files,'message'=>$message . $bugs,'timestamp'=>$timestamp);
        $t_changelog = '';
        $timestamp = false;
        foreach ($revs as $rev) {
            if (!array_key_exists($rev,$change_logs[$b_module])) {
                continue;
            }
            if ($timestamp === false) {
                $timestamp = strftime("%a, %d %B %Y %H:%M:%S %z",strtotime($change_logs[$b_module][$rev]['timestamp']));
            }
            $t_changelog .=  
                '  [' . trim($change_logs[$b_module][$rev]['committer']) ."]\n" . 
                rtrim(preg_replace("/\n\n+/","\n",$change_logs[$b_module][$rev]['message'] )). "\n    " . trim($change_logs[$b_module][$rev]['timestamp']) . "\n";
        }
        if (!$t_changelog) {
            $first = false;
            continue;
        }
        if ($first) {
            $newly_changed = true;
            $first = false;
        }
        $change_header = $package_names[$module] ." ($release) stable; urgency=low\n\n";
        $changelog .= $change_header . $t_changelog . "\n --  Carl Leitner <litlfred@ibiblio.org>  $timestamp\n\n";
    }
    if  (!$changelog) {
        I2CE::raiseError("Cannot produce changelog for $module");
        continue;
    }    
    if (!array_key_exists($top_module,$changed_modules)) {
        $changed_modules[$top_module] = array();
    }
    $changed_modules[$top_module][$module]=array('changelog'=>$changelog, 'version'=>$version,'newly_changed'=>$newly_changed,'top_module'=>$top_module );
}



/////////////////END CHANGED


$only_new = null;
        
I2CE::raiseError("Producing deb sources");
$deb_src_dirs = array();
foreach ($changed_modules as $top_module=>$changed_mods) {
    foreach ($changed_mods as $module=>$data) {
        $version  = false;
        if (!$storage->setIfIsSet($version,"/config/data/$module/version")) {
            I2CE::raiseError("Cannot get version for $module.  Skipping");
            continue;
        }
        $config_file = false;
        if ( ! $storage->setIfIsSet($config_file,"/config/data/$module/file")) {
            I2CE::raiseError( "No config file for  $module -- Skipping");
            continue;
        }
        $config_dir = rtrim(dirname($config_file),'/');

        if (!array_key_exists($module,$branch_modules)) {
            I2CE::raiseError("Dont knwo which branch $module is in");
            continue;
        }
        $d_base_dir = $deb_src_dir . '/' . $package_names[$module] . '_' .$version;
        if (is_dir($d_base_dir) && !is_writable($d_base_dir)) {
            sudo("chown $whoami:$whoami $d_base_dir");
            if (!is_writable($d_base_dir)) {
                I2CE::raiseError("Cannot write to existing $d_base_dir");
                continue;
            }

        }

        
        $deb_options_dir = $config_dir . '/packaging/DEBIAN';
        $deb_options_file = $deb_options_dir . '/config.php';
        $recommends = array();
        $depends = array();
        $scripts = array();
        $conf_files =array();
        $database = false;
        if (is_readable($deb_options_file)) {
            I2CE::raiseError("Loading $deb_options_file for $module");
            include_once($deb_options_file);
        }

        $depends[] = 'php5 (>= 5.2.6)';
        
        $b_module = $branch_modules[$module];
        $prefix  ='';
        if ($b_module != $top_module) {
            //it is a site module and there fore the prefix needs to change for the file
            $t_config_file = false;
            if ( ! $storage->setIfIsSet($t_config_file,"/config/data/$top_module/file")) {
                I2CE::raiseError( "No config file for top module $top_module -- Skipping");
                continue;
            }
            $t_config_dir = rtrim(dirname($t_config_file),'/');
            
            $b_config_file = false;
            if ( ! $storage->setIfIsSet($b_config_file,"/config/data/$b_module/file")) {
                I2CE::raiseError( "No config file for branch module $b_module -- Skipping");
                continue;
            }
            $b_config_dir = rtrim(dirname($b_config_file),'/');
            $prefix = substr($t_config_dir,strlen($b_config_dir) +1) .'/';
        }
        
        if ( ! $storage->setIfIsSet($config_file,"/config/data/$module/file")) {
            I2CE::raiseError( "No config file for $module -- Skipping");
            continue;
        }

        if (!$data['newly_changed']) {
            if ($booleans['only-new'] || ($booleans['prompt'] && prompt("Generate only newly changed module ($module)?",$only_new))) {
                continue;
            }
        }
     
        $deb_src_dirs[$module] = $d_base_dir;
        $d_dir = $d_base_dir . '/DEBIAN';
        exec("mkdir -p $d_dir");
        $doc_dir = "$d_base_dir/usr/share/doc/" . $package_names[$module] ;
        exec("mkdir -p $doc_dir");
        $c_file = $d_base_dir .'/usr/share/doc/' . $package_names[$module] . '/changelog.gz'; //not upstream source so it is not changelog.Debian.gz
        file_put_contents($c_file,gzencode($data['changelog'],9));


        $c_file = $d_base_dir .'/usr/share/doc/' . $package_names[$module] . '/copyright';
        $copyright = "This package was debianized by Carl Leitner <litlfred@ibiblio.org>\n" . gmdate(DATE_RFC822) . "\n\n"
            ."It was downloaded from: https://code.launchpad.net/" . $branch_modules[$module]. " \n\n"
            ."Upstream Author(s): Carl Leitner <litlfred@ibiblio.org>, Luke Duncan <lduncan@intrahealth.org>\n\n"
            ."Copyright:\n\tCopyright (C) 2007,2008,2009,2010 by Intrahealth International, Inc. \n\n"
            ."License:\n\tGPL-3\n"
            ."This program is free software; you can redistribute it and/or modify\n"
            ."it under the terms of the GNU General Public License as published by\n"
            ."the Free Software Foundation; either version 3 of the License, or\n"
            ."(at your option) any later version.\n"
            ."This program is distributed in the hope that it will be useful,\n"
            ."but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
            ."MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n"
            ."GNU General Public License for more details.\n"
            ."You should have received a copy of the GNU General Public License along\n"
            ."with this program; if not, write to the Free Software Foundation, Inc.,\n"
            ."51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.\n"
            ."On Debian systems, the full text of the GNU General Public\n"
            ."License version 3 can be found in the file\n"
            ."`/usr/share/common-licenses/GPL-3'\n\n"
            ."Packaging:\n\tCopyright (C) 2007,2008,2009,2010 by Intrahealth International, Inc.\n\n";
        file_put_contents($c_file,$copyright);
        
        $storage->setIfIsSet($desc,"/config/data/$module/description");
        $desc = "iHRIS Module $module\n "  . implode("\n " , explode("\n",wordwrap(preg_replace("/\n\n+/","\n .\n" ,$desc))));


        $req_list = array();
        $storage->setIfIsSet($req_list,"/config/data/$module/requirement",true);
        foreach ($req_list as $m=>$reqs) {
            if (!array_key_exists($m,$package_names)) {
                I2CE::raiseError("Could not find package associated to module $m in requirements list for $module");
                continue;
            }
            $dep = $package_names[$m];
            foreach ($reqs as $req) {
                switch (strtolower($req['operator'])) {
                case 'lessthan':
                    $depends[] = "$dep (<< " . $req['version'] . ')';
                    break;
                case 'greaterthan':
                    $depends[] = "$dep (>> " . $req['version'] .')';
                    break;

                case 'atleast':
                    $depends[] = "$dep (>= " . $req['version'] . ')';
                    break;

                case 'atmost':
                    $depends[] = "$dep (<= " . $req['version'] .')';
                    break;
                default:
                    break;
                }
            }
        }


        $p_dir  = dirname(dirname($config_dir));
        while ($p_dir && array_key_exists($p_dir,$mod_dirs[$top_module])) {
            //we want to add any  parent modules as recommends
            $p_mod = $mod_dirs[$top_module][$p_dir];
            $p_dir  = dirname(dirname($p_dir));
            $dependencies =   $configurator->getDependsList($p_mod);       
            if (in_array($mod,$dependencies)) {
                //we don't this module to depend on any module that is dependent on it.
                continue;
            }
            if (!array_key_exists($p_mod,$package_names)) {
                continue;
            }
            $recommends[] = $package_names[$p_mod];
        }

        $req_list = array();
        $storage->setIfIsSet($req_list,"/config/data/$module/enable",true);
        foreach ($req_list as $m) {
            if (!array_key_exists($m,$package_names)) {
                continue;
            }
            $recommends[] = $package_names[$m];
        }

        
        
        
        $files = $versioned_files[$top_module][$module];        
        $src_dir = "$branches_dir/{$releases[$b_module]['dir']}/$prefix";
        $is_site_module =  in_array("pages/index.php",$files);
        $is_site_top_module =  in_array("pages/index.php",$versioned_files[$top_module][$top_module]);
        
        $is_within_site = ($top_module != $b_module) || $is_site_module || $is_site_top_module;


        if (!$is_within_site) {
            $tgt_dir = $d_base_dir .'/usr/lib/iHRIS/lib/' . $releases[$b_module]['sub_major_version'] ."/" . $top_module;
        } else {
            I2CE::raiseError("$module is within site $top_module");
            $tgt_dir = $d_base_dir .'/var/lib/iHRIS/sites/' . $top_module;
        }

        exec("mkdir -p $tgt_dir");


        if ($is_site_module) {
            $htaccess_file =  "pages/htaccess.TEMPLATE";
        } else {
            $htaccess_file = false;
        }
        $has_rewrite = false;
        $apache_conf_file = false;
        foreach  ($files as $file) {            
            if ($file == 'packaging' || substr($file,0,10) ==  'packaging/') {
                continue;
            }
            $src_file = $src_dir .$file;
            if (is_dir($src_file)) {            
                exec(" mkdir -p " . escapeshellarg("$tgt_dir/$file"));
            } else if ($file == $htaccess_file) {
                $tgt_file = $d_base_dir. "/etc/apache2/conf.d/$module.conf";
                $apache_conf_file = "/etc/apache2/conf.d/$module.conf";
                $conf_files[] = $apache_conf_file;
                $htaccess= file_get_contents($src_file);
                $has_rewrite = preg_match('/^\s*RewriteBase\s+(.+)\s*/m',$htaccess,$matches);
                if ($has_rewrite) {
                    $pages = '/var/lib/iHRIS/sites/' . $top_module . '/pages';
                    $rewrite = $matches[1];
                    $htaccess = "Alias $rewrite $pages\n<Directory $pages>\nDirectoryIndex index.php\n$htaccess\n</Directory>\n";
                    exec("mkdir -p $d_base_dir/etc/apache2/conf.d/");
                    file_put_contents($tgt_file,$htaccess);
                } else {
                    I2CE::raiseError("No RewriteBase in $htaccess");
                }
            } else  {
                $tgt_file = "$tgt_dir/$file";
                $dest_d = explode("/", $tgt_file);
                array_pop($dest_d);
                $dest_d = implode("/",$dest_d);
                exec( "mkdir -p " . escapeshellarg($dest_d) );
                exec("cp " . escapeshellarg($src_file) . ' ' . escapeshellarg($tgt_file));

            }
        }


        if ($is_site_module && $has_rewrite) {
            I2CE::raiseError("Creating site for $module");
            $local_config_file = "pages/local/config.values.php";
            if (!$database) {
                $database =  preg_replace('/[^a-zA-Z0-9_]/','_',$module . '_' . $releases[$b_module]['sub_major_version']);
            }
            $local_config = '<?php
$i2ce_site_i2ce_path = "/usr/lib/iHRIS/lib/' . $releases['I2CE']['sub_major_version'] . '/I2CE";  
$i2ce_site_dsn = "mysql://ihris:manage@unix(/var/run/mysqld/mysqld.sock)/' .$database . '" ;
$i2ce_site_module_config = "/var/lib/iHRIS/sites/' . $module . '/' . basename($config_file) . '";
';
            $config_tgt = $tgt_dir . '/' . $local_config_file;
            $config_dir = explode("/", $config_tgt);
            array_pop($config_dir);
            $config_dir = implode("/" , $config_dir);
            exec("mkdir -p $config_dir");
            file_put_contents($config_tgt, $local_config);
            $mysql_dir = $d_base_dir . '/usr/share/dbconfig-common/data/' . $package_names[$module] .'/install-dbadmin';
            exec("mkdir -p $mysql_dir");
            $mysql_file = $mysql_dir . "/mysql";
            file_put_contents($mysql_file,"SET GLOBAL log_bin_trust_function_creators = 1;"); //needs to be run as admin
            exec("mkdir -p " . $d_base_dir. "/etc/ihris/" . $package_names[$module]);
            $depends[] =  'libapache2-mod-php5';
            $depends[] = 'debconf (>= 0.5) | debconf-2.0';
            $depends[] = 'dbconfig-common';
            $depends[] = 'php-mdb2' ;
            $depends[] = 'php-mdb2-driver-mysql';
            $recommends[] = 'php-apc (>= 3.1.6) | php5-apc (>= 3.1.6)'  ;
            $recommends[] = 'memcached';
            $recommends[] = ' php5-memcached';
            if (!array_key_exists('config',$scripts) || !$scripts['config']) {
                I2CE::raiseError("Creating config script for $module");
                $scripts['config'] = '#!/bin/sh 
# config script for ' . $package_names[$module] . '
set -e

. /usr/share/debconf/confmodule
. /usr/share/dbconfig-common/dpkg/config.mysql
dbc_dbtype="mysql"
dbc_dbuser="ihris"
dbc_dbpass="manage"
dbc_dbname="' . $database . '"


dbc_go ' . $package_names[$module] . ' $@ 
exit 0
';
            }
            if (!array_key_exists('postinst',$scripts) || !$scripts['postinst']) {
                I2CE::raiseError("Creating postinst script for $module");
                $scripts['postinst'] = '#!/bin/sh
# postinst script for ' . $package_names[$module] . '
set -e


. /usr/share/debconf/confmodule
. /usr/share/dbconfig-common/dpkg/postinst.mysql


dbc_go ' . $package_names[$module] . ' $@ 

a2enmod rewrite

if [ -f /etc/init.d/apache2 ] ; then
       if [ -x /usr/sbin/invoke-rc.d ]; then
                invoke-rc.d apache2 reload 3>/dev/null || true
              else
                /etc/init.d/apache2 reload 3>/dev/null || true
             fi
fi
if [ -f /etc/init.d/memcached ] ; then
             if [ -x /usr/sbin/invoke-rc.d ]; then
                invoke-rc.d memcached restart 3>/dev/null || true
              else
                /etc/init.d/memcached restart 3>/dev/null || true
            fi
fi
exit 0
';
            }
        }

        
        foreach (array("config","postinst","preinst","postrm","prerm") as $script) {
            $deb_options_script = $deb_options_dir . '/' . $script;
            if (is_readable($deb_options_script)) {
                $scripts[$script] = file_get_contents($deb_options_script); //overrides what was done above!
            }
            if (array_key_exists($script,$scripts) && $scripts[$script]) {
                $tgt_script_file = $d_dir .'/' . $script;
                I2CE::raiseError("Putting $tgt_script_file");
                file_put_contents($tgt_script_file,$scripts[$script]);
                exec("chmod 755 $tgt_script_file");
            }
        }
        
        if (count($conf_files) > 0) {
            $conf_file = $d_dir . '/conffiles';
            file_put_contents($conf_file, implode("\n",$conf_files)  . "\n");
        }
        
        if (count($recommends) > 0) {
            $recommends = "Recommends: " . implode(",",$recommends) . "\n";
        } else {
            $recommends = '';
        }

        $c_file = $d_dir .'/control';
        $control = 
            "Package: " . $package_names[$module] ."\n"
            ."Source: " . $package_names[$module] ."\n"
            ."Version: $version\n"
            ."Section: web\n"
            ."Priority: optional\n"
            ."Maintainer: Carl Leitner <litlfred@ibiblio.org>\n"
//            ."Standards-Version: 3.9.1.0\n"
            ."Architecture: all\n"
            ."Depends: " . implode(",",$depends) . "\n"
            .$recommends
            ."Description: $desc\n";
        
        file_put_contents($c_file,$control);
        //$last_dir = getcwd();
        //chdir($d_base_dir);
        //exec("find . -type f  ! -regex '.*?debian-binary.*' ! -regex '.*?DEBIAN.*'  -print0 | xargs -0  md5sum > DEBIAN/md5sums");
        //chdir($last_dir);
    }
}


I2CE::raiseError("Building newly created packages  under $deb_src_dir");

foreach ($deb_src_dirs as $dir) {
    sudo_exec("chown -R root:root $dir");
    sudo_exec( "dpkg -b $dir $deb_bin_dir/". basename($dir) . '_all.deb');
}


chdir($deb_bin_dir);
I2CE::raiseError("Making $deb_bin_dir/Packages.gz");
sudo_exec("dpkg-scanpackages . /dev/null | gzip -9c > Packages.gz");

