<?php
/*
 * © Copyright 2007, 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Translate Templates
 *
 * @package I2CE
 * @subpackage Core
 * @author Carl Leitner <litlfred@ibiblio.org>
 * @copyright Copyright &copy; 2008, 2008 IntraHealth International, Inc. 
 * @version 1.0
 */



/*****************************************
 *
 *  Launchpad webservice helper function
 *
 *****************************************/
require_once( dirname(__FILE__) .  DIRECTORY_SEPARATOR . '..' .  DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR .'I2CE.php');
require_once( dirname(__FILE__) . DIRECTORY_SEPARATOR . 'CLI.php');
require_once (  dirname(__FILE__) . "/snoopy/Snoopy.class.php");

$launchpadapi_url ="https://api.launchpad.net/1.0/";
$launchpad_snoopy = null;
$create_redirects = true;
$oauth = false;

function launchpadLogin() {
    global $oauth;
    global $launchpad_snoopy;
    global $launchpadapi_url;
    while (!is_array($oauth)) {
        if (!$launchpad_snoopy instanceof Snoopy) {
            $launchpad_snoopy = new Snoopy();
        }

        $consumer_key = 'ihris-automated-release' ; 
        //https://help.launchpad.net/API/SigningRequests
        //first get a request token
        $launchpad_snoopy->curl_path = trim(`which curl`); //needed for https
        $post =array(
            'oauth_consumer_key'=>$consumer_key ,
            'oauth_signature_method'=> "PLAINTEXT" ,
            'oauth_signature'=> "&"
            );
        if(!@$launchpad_snoopy->submit("https://launchpad.net/+request-token",$post)) {
            I2CE::raiseError("Could not get oauth " . $launchpad_snoopy->error);
            return false;
        }
        $res = array();
        parse_str($launchpad_snoopy->results,$res);
        if (!array_key_exists('oauth_token',$res) || !array_key_exists('oauth_token_secret',$res)) {
            I2CE::raiseError("Could not get proper oauth");
        }
        while (!simple_prompt("Have you given this program authority by visting?     https://edge.launchpad.net/+authorize-token?oauth_token=" . $res['oauth_token'] ."\n")) {
            echo "Then please do so\n";
        }
        I2CE::raiseError( "You can revoke authorization by: https://edge.launchpad.net/~<<USERNAME>>/+oauth-tokens");
        $post =array(
            'oauth_token'=>$res['oauth_token'],
            'oauth_consumer_key'=>$consumer_key,
            'oauth_signature_method'=> "PLAINTEXT" ,
            'oauth_signature'=>'&' . $res['oauth_token_secret']
            );        
        if (!$launchpad_snoopy->submit('https://launchpad.net/+access-token',$post)) {
            I2CE::raiseError("Could not get access token " . $launchpad_snoopy->error);
            return false;
        }
        $res = array();
        parse_str($launchpad_snoopy->results,$res);
        if (!array_key_exists('oauth_token',$res) || !array_key_exists('oauth_token_secret',$res)) {
            I2CE::raiseError("Could not get proper oauth");
            $res = false;
            return false;
        }
        $oauth=array(
            'oauth_token'=>$res['oauth_token'],
            'oauth_consumer_key'=>$consumer_key,
            'oauth_signature_method'=> "PLAINTEXT" ,
            'oauth_signature'=>'&' . $res['oauth_token_secret'],
            'oauth_version'=>"1.0"
            );
    }
    return true;
}


function launchpadPrep($post) {
   global $oauth;
   $post = array_merge($oauth,$post);
   $post['oauth_nonce'] = rand(1,32768);
   $post['oauth_timestamp'] = time();
   return $post;
}

function launchpadFetch($url_append,$post=array(),$type ='json') {
    global $launchpad_snoopy;
    global $launchpadapi_url;
    if (!launchpadLogin()) {
        I2CE::raiseError("Could not login to launchpad");
        return false;
    }
    if (! ($launchpad_snoopy->fetch($launchpadapi_url.$url_append,launchpadPrep($post)))) {
        I2CE::raiseError("Error on $url_append " . $launchpad_snoopy->error);
        return false;
    }
    return launchpadResults($type);
}


function launchpadSubmit($url_append,$post=array(),$files =array(),$type ='json'){
    global $launchpad_snoopy;
    global $launchpadapi_url;
    if (!launchpadLogin()) {
        I2CE::raiseError("Could not login to launchpad");
        return false;
    }
    if (count($files) > 0) {
        $launchpad_snoopy->set_submit_multipart();        
    }
    if (! ($launchpad_snoopy->submit($launchpadapi_url.$url_append,launchpadPrep($post),$files))) {
        $launchpad_snoopy->set_submit_normal();
        I2CE::raiseError("Error on $url_append " . $launchpad_snoopy->error);
        return false;
    }
    $launchpad_snoopy->set_submit_normal();
    return launchpadResults($type);
}

function launchpadResults($type) {
    global $launchpad_snoopy;
    switch ($type) {
    case 'code':
        return $launchpad_snoopy->response_code;
    case 'json':
        return json_decode($launchpad_snoopy->results,true);
    case 'parse':
        $res = array();
        parse_str($launchpad_snoopy->results,$res);
        return $res;
    default:
        return $launchpad_snoopy->results;
    }
}


function getProjectDetails($project,$series) {    
    if (! $res =launchpadFetch( $project .'/' . $series)) {
        return false;
    }
    return $res;
}
function getReleaseDetails($project,$series,$version) {
   //https://api.edge.launchpad.net/1.0/<project.name>/<project_series.name>/<release.version>
    if (! $res =launchpadFetch( $project .'/' . $series . '/' . $version)) {
        return false;
    }
    return $res;

}

function createRelease($project,$series,$version,$desc) {
    global $launchpad_snoopy;
    $milestone_url = $project .'/+milestone/' .$version;
    if ( !is_array( $res =launchpadFetch( $milestone_url ))) {
        if (!simple_prompt("Milestone $version has not been created for $project.  Create?")) {
            return false;
        }
        $post = array(
            'ws.op'=>'newMilestone',
            'name'=>$version,
            'description'=>$desc
            );
        if ( substr( $res =launchpadSubmit( $project .'/' . $series  ,$post,array(),'code'),0,strlen('HTTP/1.1 201')) != 'HTTP/1.1 201') {
            I2CE::raiseError("Could not create milestone to $project/$series/$version release [$res]\n" . $launchpad_snoopy->results);
            return false;
        }
    }
    $post = array(
        'ws.op'=>'createProductRelease',
        'date_released'=>strftime('%Y-%m-%dT11:22:33Z'),
        'milestone'=>$version,
        'name'=>$version,
        'description'=>$desc
        );
    print_r($post);
    if ( substr( $res =launchpadSubmit($milestone_url ,$post,array(),'code'),0,strlen('HTTP/1.1 201')) != 'HTTP/1.1 201') {
        I2CE::raiseError("Could not create milestone to $project/$series/$version release [$res]\n" . $launchpad_snoopy->results);
        return false;
    }
    return true;
}


function postRelease($tar_ball,$project,$series,$version,$desc) {
    global $launchpad_snoopy;
    if (!is_readable($tar_ball)) {
        I2CE::raiseError("Cannot read $tar_ball");
        return false;
    }
    $post = array(
        'ws.op'=>'add_file',
        'content_type'=>'application/octet-stream',
        'description'=>$desc,
        //'file_content'=>file_get_contents($tar_ball),
        //'file_content'=>"data:application/octet-stream;base64,\n" . base64_encode(file_get_contents($tar_ball)),
        'file_type'=>'Code Release Tarball',
        'filename'=>basename($tar_ball),
        );    
    $files = array('file_content'=>$tar_ball);
    if ( substr($res =launchpadSubmit( $project .'/' . $series . '/' . $version,$post,$files,'code'),0,strlen('HTTP/1.1 201')) != 'HTTP/1.1 201') {
        I2CE::raiseError("Could not post " . basename($tar_ball)  . " to $project/$series/$version release [$res]\n" . $launchpad_snoopy->results);
        print_r($launchpad_snoopy);
        return false;
    }
    return true;
}